#include <FastLED.h>
#include <ESP8266WiFi.h>
#include <PubSubClient.h>
#include "credentials.h" 


// Update these with values suitable for your network.
#define NUM_LEDS 1
#define DATA_PIN 02

const char* ssid_name = SSID_NAME;
const char* ssid_pass = SSID_PASS;
const char* mqtt_server = MQTT_SERVER;
const char* mqtt_user = MQTT_USER;
const char* mqtt_pass = MQTT_PASS;
const char* mqtt_client = MQTT_CLIENT;

const char* mqtt_subTopic = "Ironman/notification/";
const char* mqtt_pubTopic = "SomeTopic";

int x;                //number of blinks
int red;              //0-255 red intensity
int green;            //0-255 green intensity
int blue;             //0-255 blue intsnsity
int ledOff;           //delay between blinks ms
int ledOn;            //led on times  ms
int constLoop;        //one off vs constant blinks 0/1
int ledBrightness;    //brightness 0-255 


WiFiClient espClient;
PubSubClient client(espClient);
// Define the array of leds
CRGB leds[NUM_LEDS];

void setup() {
  Serial.begin(115200);
  setup_wifi();
  client.setServer(mqtt_server, 1883);
  client.setCallback(callback);
  FastLED.addLeds<WS2812B, DATA_PIN, GRB>(leds, NUM_LEDS); 
       
}

String getValue(String data, char separator, int index){
    int found = 0;
    int strIndex[] = { 0, -1 };
    int maxIndex = data.length() - 1;

    for (int i = 0; i <= maxIndex && found <= index; i++) {
        if (data.charAt(i) == separator || i == maxIndex) {
            found++;
            strIndex[0] = strIndex[1] + 1;
            strIndex[1] = (i == maxIndex) ? i+1 : i;
        }
    }
    return found > index ? data.substring(strIndex[0], strIndex[1]) : "";
}



void setup_wifi() {

  delay(10);
  // We start by connecting to a WiFi network
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid_name);

  WiFi.begin(ssid_name, ssid_pass);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
    
  }
    Serial.println("");
    Serial.println("WiFi connected");
    Serial.println("IP address: ");
    Serial.println(WiFi.localIP());
}

void callback(char* topic, byte* payload, unsigned int length) {
  
  String messageTemp;
  for (int i = 0; i < length; i++) 
    {messageTemp += (char)payload[i];}

//splitting up the MQTT message into variables
  String tempNumber = getValue(messageTemp,':',0);
  x = tempNumber.toInt();
//RGB
  String tempRed = getValue(messageTemp,':',1);
  red = tempRed.toInt();  
  String tempGreen = getValue(messageTemp,':',2);
  green = tempGreen.toInt();  
  String tempBlue = getValue(messageTemp,':',3);
  blue = tempBlue.toInt();
//Timing
  String tempLedOff = getValue(messageTemp,':',4);
  ledOff = tempLedOff.toInt();
  String tempLedOn = getValue(messageTemp,':',5);
  ledOn = tempLedOn.toInt();
//loop mode and brightness  
  String tempBrightness = getValue(messageTemp,':',6);
  ledBrightness = tempBrightness.toInt();
  String tempConstant = getValue(messageTemp,':',7);
  constLoop = tempConstant.toInt();

  

// loop blinks limited number of times    
if (constLoop == 0){ 
  for ( int i=0; i < x; i++){ 
      FastLED.setBrightness(ledBrightness);
      leds[0] = CRGB::Black;
      FastLED.show();
      delay(ledOff);
      leds[0].setRGB(red,green,blue);
      FastLED.show();
      delay(ledOn);      
     }
  }

//clear notification
if (constLoop == 1){ 
      leds[0] = CRGB::Black;
      FastLED.show();
      delay(ledOff);
      }    

}

void reconnect() {
  // Loop until we're reconnected
  while (!client.connected()) {
    Serial.print("Attempting MQTT connection...");
    // Attempt to connect
    if (client.connect(mqtt_client)) {
      Serial.println("connected");
      client.subscribe(mqtt_subTopic);
    } else {
      Serial.print("failed, rc=");
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds");
                            
      // Wait 5 seconds before retrying
      delay(5000);
    }
  }
}

void loop() { 
  if (!client.connected()) {
    reconnect();
  }
  client.loop();
  if (constLoop == 2){
      FastLED.setBrightness(ledBrightness);
      leds[0] = CRGB::Black;
      FastLED.show();
      delay(ledOff);
      leds[0].setRGB(red,green,blue);
      FastLED.show();
      delay(ledOn);
    }
}
